@@ -1,22 +1,20 @@
 package HTML::FormWidgets::Cloud;
 
-# @(#)$Id: Cloud.pm 83 2008-09-24 00:27:50Z pjf $
+# @(#)$Id: Cloud.pm 135 2009-02-19 17:51:07Z pjf $
 
 use strict;
 use warnings;
-use base qw(HTML::FormWidgets);
+use parent qw(HTML::FormWidgets);
 
-use version; our $VERSION = qv( sprintf '0.2.%d', q$Rev: 83 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.3.%d', q$Rev: 135 $ =~ /\d+/gmx );
 
 __PACKAGE__->mk_accessors( qw(data js_obj) );
 
-sub init {
+sub _init {
    my ($self, $args) = @_;
 
    $self->data(   {} );
-   $self->js_obj( q(tableObj.liveGrid) );
-
-   $self->NEXT::init( $args );
+   $self->js_obj( q(behaviour.table.liveGrid) );
    return;
 }
 
