@@ -1,23 +1,21 @@
 package HTML::FormWidgets::Note;
 
-# @(#)$Id: Note.pm 83 2008-09-24 00:27:50Z pjf $
+# @(#)$Id: Note.pm 135 2009-02-19 17:51:07Z pjf $
 
 use strict;
 use warnings;
-use base qw(HTML::FormWidgets);
+use parent qw(HTML::FormWidgets);
 
-use version; our $VERSION = qv( sprintf '0.2.%d', q$Rev: 83 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.3.%d', q$Rev: 135 $ =~ /\d+/gmx );
 
 __PACKAGE__->mk_accessors( qw(width) );
 
-sub init {
+sub _init {
    my ($self, $args) = @_;
 
    $self->container( 0 );
    $self->sep(       q() );
    $self->width(     undef );
-
-   $self->NEXT::init( $args );
    return;
 }
 
@@ -28,7 +26,7 @@ sub _render {
    $args->{style} .= 'text-align: '.$self->align.q(;) if ($self->align);
    $args->{style} .= ' width: '.$self->width.q(;)     if ($self->width);
 
-   ($text = $self->msg( $self->name ) || $self->text) =~ s{ \A \n }{}msx;
+   ($text = $self->text || $self->loc( $self->name )) =~ s{ \A \n }{}msx;
 
    return $self->hacc->div( $args, $text );
 }
