@@ -1,42 +1,45 @@
 package HTML::FormWidgets::Table;
 
-# @(#)$Id: Table.pm 94 2008-09-27 21:48:32Z pjf $
+# @(#)$Id: Table.pm 137 2009-02-22 02:41:15Z pjf $
 
 use strict;
 use warnings;
-use base qw(HTML::FormWidgets);
+use parent qw(HTML::FormWidgets);
 
-use version; our $VERSION = qv( sprintf '0.2.%d', q$Rev: 94 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.3.%d', q$Rev: 137 $ =~ /\d+/gmx );
 
 __PACKAGE__->mk_accessors( qw(add_tip assets data edit hide js_obj
                               remove_tip select) );
 
-sub init {
+my $TTS = q( ~ );
+
+sub _init {
    my ($self, $args) = @_; my $text;
 
    $text  = 'Enter a new item into the adjacent text ';
    $text .= 'fields and then click this button to add ';
    $text .= 'it to the list';
-   $self->add_tip(    $self->msg( q(tableAddTip) ) || $text );
-   $self->assets(     q() );
-   $self->class(      q(small table) );
-   $self->container(  0 );
-   $self->data(       { flds => [], values => [] } );
-   $self->edit(       0 );
-   $self->hide(       [] );
-   $self->js_obj(     q(tableObj) );
+   $text  = $self->loc( q(tableAddTip) ) || $text;
+   $self->add_tip(      $self->hint_title.$TTS.$text );
+   $self->assets(       q() );
+   $self->class(        q(small table) );
+   $self->container(    0 );
+   $self->data(         { flds => [], values => [] } );
+   $self->edit(         0 );
+   $self->hide(         [] );
+   $self->js_obj(       q(behaviour.table) );
    $text  = 'Select one or more items from the ';
    $text .= 'above list and then click this button ';
    $text .= 'to remove them';
-   $self->remove_tip( $self->msg( q(tableRemoveTip) ) || $text );
-   $self->select(     q() );
-
-   $self->NEXT::init( $args );
+   $text  = $self->loc( q(tableRemoveTip) ) || $text;
+   $self->remove_tip(   $self->hint_title.$TTS.$text );
+   $self->select(       q() );
    return;
 }
 
 sub _render {
-   my ($self, $args) = @_; my ($fld, $r_no, $rows, $text, $text1, $val);
+   my ($self, $args) = @_;
+   my ($class, $fld, $fld_val, $r_no, $rows, $text, $text1, $type, $val);
 
    my $c_no  = 0;
    my $cells = q();
@@ -75,29 +78,15 @@ sub _render {
       $cells = q(); $c_no = 0;
 
       if ($self->select eq q(left) and $data->{values}->[0]) {
-         $cells .= $self->_check_box( $r_no, $c_no, $val );
+         $cells .= $self->_check_box( $r_no, $c_no, $val->{id} );
       }
 
       for $fld (@{ $data->{flds} }) {
          if ($self->edit) {
             $args            = {};
             $args->{default} = $val->{ $fld };
-
-            if (exists $data->{maxlengths}->{ $fld }) {
-               $args->{maxlength} = $data->{maxlengths}->{ $fld };
-            }
-
-            $args->{name} = $self->name.q(_).$fld.$r_no;
-            $args->{size} = exists $data->{sizes}->{ $fld }
-                          ? $data->{sizes}->{ $fld } : 10;
-            $text         = $hacc->textfield( $args );
-            $args         = { class => q(dataValue) };
-
-            unless (exists $data->{wrap}->{ $fld }) {
-               $args->{class} .= q( nowrap);
-            }
-
-            $cells .= $hacc->td( $args, $text );
+            $args->{name}    = $self->name.q(_).$fld.$r_no;
+            $cells          .= $self->_editable_cell( $data, $fld, $args );
          }
          else {
             next if ($data->{hclass}->{ $fld }
@@ -106,50 +95,49 @@ sub _render {
             $args          = {};
             $args->{align} = exists $data->{align}->{ $fld }
                            ? $data->{align}->{ $fld } : q(left);
+            $class         = $data->{class} || q(dataValue);
 
-            if ($val->{class} and exists $val->{class}->{ $fld }) {
-               $args->{class}  = $val->{class}->{ $fld };
-            } else {
-               $args->{class}  = $c_no % 2 == 0 ? q(even) : q(odd);
-               $args->{class} .= q( ).($data->{class} || q(dataValue));
+            if (ref $class and exists $class->{ $fld }) {
+               $args->{class} = $class->{ $fld };
             }
+            else { $args->{class} = $class }
+
+            $args->{class} .= $c_no % 2 == 0 ? q( even) : q( odd);
 
             unless (exists $data->{wrap}->{ $fld }) {
                $args->{class} .= q( nowrap);
             }
 
-            $cells .= $hacc->td( $args, $val->{ $fld } || q(&nbsp;) )."\n";
+            $fld_val = $self->inflate( $val->{ $fld } ) || q(&nbsp;);
+            $cells  .= $hacc->td( $args, $fld_val )."\n";
          }
 
          $c_no++;
       }
 
       if ($self->select eq q(right) and $data->{values}->[0]) {
-         $cells .= $self->_check_box( $r_no, $c_no, $val );
+         $cells .= $self->_check_box( $r_no, $c_no, $val->{id} );
       }
 
-      $args  = { id => $self->name.q(_row).$r_no };
+      $class = $data->{class} || q(dataValue);
+      $args  = { class => $class, id => $self->name.q(_row).$r_no };
       $rows .= $hacc->tr( $args, $cells ); $r_no++;
    }
 
-   push @{ $self->hide }, { name => $self->name.q(_nrows), value => $r_no };
+   my $content = $self->inflate( { name    => $self->name.q(_nrows),
+                                   default => $r_no,
+                                   type    => q(hidden),
+                                   widget  => 1 } );
+   push @{ $self->hide }, { content => $content };
 
    if ($self->edit) {
       $cells = q();
 
       for $c_no (0 .. $#{ $data->{flds} }) {
-         $fld  = $data->{flds}->[ $c_no ];
-         $args = { id => $self->name.q(_add).$c_no };
-
-         if (exists $data->{maxlengths}->{ $fld }) {
-            $args->{maxlength} = $data->{maxlengths}->{ $fld };
-         }
-
-         $args->{name}  = $self->name.q(_).$fld;
-         $args->{size}  = exists $data->{sizes}->{ $fld }
-                        ? $data->{sizes}->{ $fld } : 10;
-         $text          = $hacc->textfield( $args );
-         $cells        .= $hacc->td( $text );
+         $fld          = $data->{flds}->[ $c_no ];
+         $args         = { id => $self->name.q(_add).$c_no };
+         $args->{name} = $self->name.q(_).$fld;
+         $cells       .= $self->_editable_cell( $data, $fld, $args );
       }
 
       $args             = {};
@@ -175,8 +163,10 @@ sub _render {
          $text            .= $hacc->span( $args, $text1 );
       }
 
+      $class  = $data->{class} || q(dataValue);
       $cells .= $hacc->td( $text );
-      $rows  .= $hacc->tr( { id => $self->name.q(_add) }, $cells );
+      $rows  .= $hacc->tr( { class => $class,
+                             id    => $self->name.q(_add) }, $cells );
    }
 
    return $hacc->table( { class => ($self->prompt ? q(form) : q(std)) },
@@ -185,13 +175,38 @@ sub _render {
 
 # Private methods
 
+sub _editable_cell {
+   my ($self, $data, $fld, $args) = @_;
+
+   if (exists $data->{maxlengths}->{ $fld }) {
+      $args->{maxlength} = $data->{maxlengths}->{ $fld };
+   }
+
+   my $type = $data->{typelist}->{ $fld } || q(textfield);
+
+   if ($type eq q(textarea)) {
+      $args->{rows} = exists $data->{rows}->{ $fld }
+                    ? $data->{rows}->{ $fld } : 5;
+      $args->{cols} = exists $data->{cols}->{ $fld }
+                    ? $data->{cols}->{ $fld } : 60;
+   }
+   elsif ($type eq q(textfield)) {
+      $args->{size} = exists $data->{sizes}->{ $fld }
+                    ? $data->{sizes}->{ $fld } : 10;
+   }
+
+   my $text = $self->hacc->$type( $args );
+
+   return $self->hacc->td( { class => q(dataValue) }, $text );
+}
+
 sub _check_box {
-   my ($self, $r_no, $c_no, $val) = @_; my ($text, $args);
+   my ($self, $r_no, $c_no, $id) = @_; my ($text, $args);
 
    $args = { name => $self->name.q(_select).$r_no };
-   $args->{value} = $val->{id} if ($val->{id});
+   $args->{value} = $id if ($id);
    $text = $self->hacc->checkbox( $args );
-   $args = { align => q(center), class => $c_no % 2 == 0 ? q(even) : q(odd) };
+   $args = { align => q(center), class => $c_no % 2 == 0 ? q(odd) : q(even) };
 
    return $self->hacc->td( $args, $text );
 }
