@@ -1,21 +1,19 @@
 package HTML::FormWidgets::Textfield;
 
-# @(#)$Id: Textfield.pm 83 2008-09-24 00:27:50Z pjf $
+# @(#)$Id: Textfield.pm 135 2009-02-19 17:51:07Z pjf $
 
 use strict;
 use warnings;
-use base qw(HTML::FormWidgets);
+use parent qw(HTML::FormWidgets);
 
-use version; our $VERSION = qv( sprintf '0.2.%d', q$Rev: 83 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.3.%d', q$Rev: 135 $ =~ /\d+/gmx );
 
 __PACKAGE__->mk_accessors( qw(width) );
 
-sub init {
+sub _init {
    my ($self, $args) = @_;
 
    $self->width( 40 );
-
-   $self->NEXT::init( $args );
    return;
 }
 
